/*
 * Copyright (c) 2007 Henric Jungheim <software@henric.info>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "nist_ctr_drbg.h"

#include <stdio.h>

void
nist_dump_simple_hex(const void* data, int length)
{
	int i;
	const unsigned char* p = data;
	for (i = 0; i < length; ++i)
		printf("%02X", *p++);
}

void
nist_dump_hex(const void* data, int length)
{
	int i;
	const unsigned char* p = data;
	for (i = 0; i < length; ++i) {
		if (i && !(i & 3)) {
			printf((i & 31) ? "-" : "\n");
		}
		printf("%02x", *p++);
	}
}

void
nist_dump_named_hex(const char* name, const void* data, int length)
{
	printf(" %s: ", name);
	nist_dump_hex(data, length);
	printf("\n");
}

void
nist_dump_ctr_drbg(const NIST_CTR_DRBG* drbg)
{
	printf("  reseed_counter = %d\n", drbg->reseed_counter);
	printf("  V: ");
	nist_dump_hex(&drbg->V[0], sizeof(drbg->V));
	printf("\n");
}

void
nist_dump_aes_ctx(const rijndael_ctx* ctx)
{
	printf("  Nr = %d\n  ek:\n", ctx->Nr);
	nist_dump_hex(ctx->ek, sizeof(ctx->ek));

	printf("\n");

	if (!ctx->enc_only) {
		printf("  dk:\n");
		nist_dump_hex(ctx->dk, sizeof(ctx->dk));
		printf("\n");
	}
}

